/*
 * Copyright 2020 Kunbus GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <cstdlib>
#include <cstdio>
#include "ftdi.h"

using namespace std;

/*
 * 
 */
int main(int argc, char** argv) {
    struct ftdi_context *context;
    char c;
    FILE * file;
    int ret;

    if ((context = ftdi_new()) == 0) {
        printf("ftdi_new failed\n");
        return EXIT_FAILURE;
    }

    if ((file = fopen("/sys/bus/usb/devices/1-1.5.2/devnum", "r")) == NULL) {
        printf("fopen /sys/bus/usb/devices/1-1.5.2/devnum failed\n");
        return EXIT_FAILURE;
    }

    ret=fread(&c, 1, 1, file);

    fclose(file);

    if (ret != 1) {
        printf("fread failed\n");
        return EXIT_FAILURE;
    }
    
    int devnum = c - '0'; //convert char to int
    
    char str[6] = "";
    sprintf(str, "d:1/%i", devnum);
    
    if ((ret = ftdi_usb_open_string(context, str)) != 0) {
        printf("ftdi_usb_open_string failed: %i\n", ret);
        return EXIT_FAILURE;
    }
    
    // 0xf0 = all four CBUS pins are outputs, CBUS[0] is driven low
    if ((ret = ftdi_set_bitmode(context, 0xf1, 0x20)) != 0) {
        printf("ftdi_set_bitmode failed: %i", ret);
        return EXIT_FAILURE;
    }

    ftdi_usb_close(context);
    ftdi_free(context);

    // rebind ftdi_sio driver, it was unbound when the device was opened above
    if ((file = fopen("/sys/bus/usb/drivers/ftdi_sio/bind", "w")) == NULL) {
        printf("fopen /sys/bus/usb/drivers/ftdi_sio/bind failed\n");
        return EXIT_FAILURE;
    }
    
    if ((ret=fwrite("1-1.5.2:1.0", 11, 1, file)) != 1) {
        printf("fwrite failed: %i\n",ret);
        return EXIT_FAILURE;
    }
    
    return EXIT_SUCCESS;
}

